﻿using Styx;
using Styx.CommonBot;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Frodo
{
    public static class Helpers
    {
        private const int BannerOfTheAlliance = 61573;
        private const int BannerOfTheHorde = 61574;
        public static bool IsTrainingDummy(this WoWUnit unit)
        {
            // return Lists.BossList.TrainingDummies.Contains(unit.Entry);

            int bannerId = StyxWoW.Me.IsHorde ? BannerOfTheAlliance : BannerOfTheHorde;
            return unit != null && unit.Level > 1
                && ((unit.CurrentHealth == 1 && unit.MaxHealth < unit.Level) || unit.HasAura(bannerId) || unit.Name.Contains("Training Dummy"));
        }
        /// <summary>
        /// IsBoss() checks usually appear in a sequence testing same target repeatedly.  
        /// Cache the values for a fast return in that circumstanc
        /// </summary>
        private static WoWGuid _lastIsBossGuid;
        private static bool _lastIsBossResult = false;

        public static bool IsBoss(this WoWUnit unit)
        {
            WoWGuid guid = unit == null ? WoWGuid.Empty : unit.Guid;
            if (guid == _lastIsBossGuid)
                return _lastIsBossResult;

            _lastIsBossGuid = guid;

            _lastIsBossResult = unit.IsBoss;

            if (!_lastIsBossResult)
                _lastIsBossResult = IsTrainingDummy(unit);
            return _lastIsBossResult;
        }

        /// <summary>
        /// Checks if target is a Critter that can safely be ignored
        /// </summary>
        /// <param name="u"></param>
        /// WoWUnit to check
        /// <returns>true: can ignore safely, false: treat as attackable enemy</returns>
        public static bool IsIgnorableCritter(this WoWUnit u)
        {
            if (!u.IsCritter)
                return false;

            // good enemy if BotPoi
            if (Styx.CommonBot.POI.BotPoi.Current.Guid == u.Guid && Styx.CommonBot.POI.BotPoi.Current.Type == Styx.CommonBot.POI.PoiType.Kill)
                return false;

            // good enemy if Targeting
            if (Targeting.Instance.TargetList.Contains(u))
                return false;

            // good enemy if Threat towards us
            if (u.ThreatInfo.ThreatValue != 0 && u.IsTargetingMyRaidMember)
                return false;

            // Nah, just a harmless critter
            return true;
        }
        /// <summary>
        /// Calls the UnitCanAttack LUA to check if current target is attackable. This is
        /// necessary because the WoWUnit.Attackable property returns 'true' when targeting
        /// any enemy player including in Sanctuary, not PVP flagged, etc where a player
        /// is not attackable
        /// </summary>
        public static bool CanWeAttack(this WoWUnit unit, bool restoreFocus = true)
        {
            if (unit == null)
                return false;

            bool canAttack = false;

            if (unit.Guid == StyxWoW.Me.CurrentTargetGuid)
                canAttack = Lua.GetReturnVal<bool>("return UnitCanAttack(\"player\",\"target\")", 0);
            else
            {
                // skip test if not a player 
                if (!unit.IsPlayer)
                    return true;

                WoWUnit focusSave = StyxWoW.Me.FocusedUnit;
                StyxWoW.Me.SetFocus(unit);
                canAttack = Lua.GetReturnVal<bool>("return UnitCanAttack(\"player\",\"focus\")", 0);
                if (restoreFocus)
                {
                    if (focusSave == null || !focusSave.IsValid)
                        StyxWoW.Me.SetFocus(WoWGuid.Empty);
                    else
                        StyxWoW.Me.SetFocus(focusSave);
                }
            }

            return canAttack;
        }

        /// <summary>
        /// get the effective distance between two mobs accounting for their 
        /// combat reaches (hitboxes)
        /// </summary>
        /// <param name="unitOrigin">toon originating spell/ability.  If no destination specified then assume 'Me' originates and 'unit' is the target</param>
        /// <param name="unitTarget">target of spell.  if null, assume 'unit' is target of spell cast by 'Me'</param>
        /// <returns>normalized attack distance</returns>
        public static float SpellDistance(this WoWUnit unitOrigin, WoWUnit unitTarget = null)
        {
            // abort if mob null
            if (unitOrigin == null)
                return 0;

            // when called as SomeUnit.SpellDistance()
            // .. convert to SomeUnit.SpellDistance(Me)
            if (unitTarget == null)
                unitTarget = StyxWoW.Me;

            // when called as SomeUnit.SpellDistance(Me) then
            // .. convert to Me.SpellDistance(SomeUnit)
            if (unitTarget.IsMe)
            {
                unitTarget = unitOrigin;
                unitOrigin = StyxWoW.Me;
            }

            // only use CombatReach of destination target 
            float dist = unitTarget.Location.Distance(unitOrigin.Location) - unitTarget.CombatReach;
            return Math.Max(0, dist);
        }
    }
}
