﻿// System includes
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

// Honorbuddy includes
using Styx;
using Styx.Common;
using Styx.CommonBot;
using Styx.Helpers;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Styx.Plugins;
using Styx.Pathing;
using Styx.WoWInternals.World;
using Newtonsoft.Json;

namespace Frodo
{
    public class Frodo : Styx.Plugins.HBPlugin
    {
        public const string TankRingName = "Sanctus, Sigil of the Unbroken";
        public const string HealingRingName = "Etheralus, the Eternal Reward";
        public const string AgiDpsRingName = "Maalus, the Blood Drinker";
        public const string IntDpsRingName = "Nithramus, the All-Seer";
        public const string StrDpsRingName = "Thorasus, the Stone Heart of Draenor";

        public static string settingsFile = null;
        public static FrodoSettings frodoSettings { get; set; }

        private static System.Drawing.Color infoColor = System.Drawing.Color.White;
        private static System.Drawing.Color badColor = System.Drawing.Color.Orange;
        private static System.Drawing.Color goodColor = System.Drawing.Color.Green;

        private static WoWItem currentlyEquippedRing = null;
        private static bool notifiedRingIsReady = false;
        /// <summary>
        /// This is a standard variable declare as it grants access to player specific information
        /// </summary>
        public static LocalPlayer Me = StyxWoW.Me;

        /// <summary>
        /// Set up your variables as static so you can access them from the settings form.
        /// These are called within there respective overrides below as described in each.
        /// </summary>
        public static string PluginName = "Frodo";
        public static string PluginAuthor = "SpeshulK926";
        public static Version PluginVersion = new Version(1, 0, 0, 1);
        public static string PluginSettingsButton = "Settings...";

        /// <summary>
        /// The name that you would like the user to see in the list of plugins.
        /// </summary>
        public override string Name
        {
            get { return PluginName; }
        }

        /// <summary>
        /// You can use this parameter to set your company name, or your forum name.
        /// </summary>
        public override string Author
        {
            get { return PluginAuthor; }
        }

        /// <summary>
        /// The version of your plugin
        /// </summary>
        public override Version Version
        {
            get { return PluginVersion; }
        }

        /// <summary>
        /// The caption of the settings button in the plugin window.
        /// </summary>
        public override string ButtonText
        {
            get { return PluginSettingsButton; }
        }

        /// <summary>
        /// If you wish to have a settings button in the plugin window, set this to "true"; else return "false".
        /// </summary>
        public override bool WantButton
        {
            get
            {
                // Check if settings button string is set. If string length is greater than 0, then we want the button
                if (PluginSettingsButton.Length > 0)
                {
                    return true;
                }
                else {
                    return false;
                }
            }
        }

        /// <summary>
        /// This function is called as soon as this assembly is loaded into memory, and thus a great place to initialize any events.
        /// </summary>
        public Frodo()
        {
            BotEvents.OnBotStopped += BotEvents_OnBotStopped;
            BotEvents.OnBotStarted += BotEvents_OnBotStarted;
            BotEvents.OnBotChanged += BotEvents_OnBotChanged;
            BotEvents.OnBotStartRequested += BotEvents_OnBotStartRequested;
            BotEvents.OnBotStopRequested += BotEvents_OnBotStopRequested;
            BotEvents.OnPulse += BotEvents_OnPulse;
        }

        private void BotEvents_OnPulse(object sender, EventArgs e)
        {
            if (frodoSettings.GeneralSettingDebugLogging)
                Logger.Write(infoColor, "ActuallyInCombat: " + Me.IsActuallyInCombat.ToString() + ", IsEquippedRingRead: " + IsEquippedRingReady().ToString());

            // always make sure we are in combat first.
            if (UnitsInCombatWithMe().Count() > 0 && IsEquippedRingReady())
            {
                bool useRing = false;
                if (notifiedRingIsReady == false && frodoSettings.GeneralSettingDebugLogging)
                {
                    //Logger.Write(infoColor, "Ring is ready to use!");
                    //notifiedRingIsReady = true;
                }

                #region tank logic
                // tanking first
                if (IsTankSpec())
                {
                    // if use on cooldown; always use it
                    if (frodoSettings.TankSettingUseOnCooldown) { useRing = true; }

                    // if we have nearby units in combat set and the limit is reached, let's use it.
                    if (frodoSettings.TankSettingNearbyUnitsInCombat > 0
                        && NearbyUnitsInCombatWithMe.Count() >= frodoSettings.TankSettingNearbyUnitsInCombat)
                    {
                        useRing = true;
                    }
                    
                    // if we have tank health percent enabled and we drop below, let's use it.
                    if (frodoSettings.TankSettingHealthPercent > 0
                        && Me.HealthPercent < frodoSettings.TankSettingHealthPercent)
                    {
                        useRing = true;
                    }

                    // if we have use on boss enabled and healing percent, let's use it on a boss.
                    if ((frodoSettings.TankSettingUseOnBoss && frodoSettings.TankSettingHealthPercent > 0)
                        && (Me.CurrentTarget.IsBoss() && Me.HealthPercent < frodoSettings.TankSettingHealthPercent))
                    {
                        useRing = true;
                    }

                    // if we have use on boss enabled and healing percent disabled, let's use on CD on boss.
                    if ((frodoSettings.TankSettingUseOnBoss && frodoSettings.TankSettingHealthPercent <= 0) 
                        && (Me.CurrentTarget.IsBoss()))
                    {
                        useRing = true;
                    }
                }
                #endregion

                #region heal logic
                if (IsHealSpec())
                {
                    if (frodoSettings.HealSettingUseOnCooldown)
                    {
                        useRing = true;
                    }
                }

                #endregion

                #region dps logic
                if (IsAgiDpsSpec() || IsIntDpsSpec() || IsStrengthDpsSpec())
                {
                    // USE IT ON COOLDOWN
                    if (frodoSettings.DpsSettingUseOnCooldown) { useRing = true; }

                    // use it when nearby units in combat is enabled and the count is met.
                    if (frodoSettings.DpsSettingNearbyUnitsInCombat > 0
                        && NearbyUnitsInCombatWithMe.Count() >= frodoSettings.DpsSettingNearbyUnitsInCombat)
                    {
                        useRing = true;
                    }

                    // use it on bosses.
                    if ((frodoSettings.DpsSettingUseOnBoss)
                        && (Me.CurrentTarget.IsBoss()))
                    {
                        useRing = true;
                    }

                }
                #endregion  

                if (useRing)
                {
                    currentlyEquippedRing.Use();
                    notifiedRingIsReady = false;
                }

            }
        }

        #region Other Junk
        
        /// <summary>
        /// This gets called shortly after the function using the same name as your plugin class is called.
        /// </summary>
        static public void Initialize()
        {
            
        }

        /// <summary>
        /// Honorbuddy core sends a Pulse event which occurs similar to a heartbeat. You can use this when you need to work with events as they happen to the player.
        /// </summary>
        public override void Pulse()
        {

        }

        /// <summary>
        /// When the user clicks on your "config" button
        /// </summary>
        public override void OnButtonPress()
        {
            Settings frmSettings = new Settings();
            frmSettings.ShowDialog();
        }

        /// <summary>
        /// Fires when the user enables your plugin (checks it)
        /// </summary>
        public override void OnEnable()
        {
            frodoSettings = LoadSettings();
            Logger.Write(infoColor, "Enabled!");

            Logger.Write(infoColor, "Current Rings:", "");
            Logger.Write(infoColor, Me.Inventory.Equipped.Finger1?.Name);
            Logger.Write(infoColor, Me.Inventory.Equipped.Finger2?.Name);

            UpdateCurrentlyEquippedRing();
            if (IsRingRightForSpec() == false)
            {
                Logger.Write(badColor, String.Format("Your ring: {0} is incorrect for your spec: {1}", currentlyEquippedRing.Name, Me.Specialization.ToString()));
            }
        }

        /// <summary>
        /// Fires when the user disables your plugin (unchecks it)
        /// </summary>
        public override void OnDisable()
        {
            Logger.Write(infoColor, "Disabled!");
            notifiedRingIsReady = false;
        }

        /// <summary>
        /// This event is called if the user changes bot bases.
        /// </summary>
        /// <param name="args"></param>
        static void BotEvents_OnBotChanged(BotEvents.BotChangedEventArgs args)
        {

        }

        /// <summary>
        /// This event occurs after the user clicks start, but before everything has been initialized.
        /// </summary>
        /// <param name="args"></param>
        static void BotEvents_OnBotStartRequested(BotEvents.BotStartStopRequestEventArgs args)
        {

        }

        /// <summary>
        /// Fired after everything has been initialized and the bot is started.
        /// </summary>
        /// <param name="args"></param>
        static void BotEvents_OnBotStarted(EventArgs args)
        {
            UpdateCurrentlyEquippedRing();
            if (IsRingRightForSpec() == false)
            {
                Logger.Write(badColor, String.Format("Your ring: {0} is incorrect for your spec: {1}", currentlyEquippedRing.Name, Me.Specialization.ToString()));
            }
        }

        /// <summary>
        /// Called when the user pushes stop, but before the bot has stopped
        /// </summary>
        /// <param name="args"></param>
        static void BotEvents_OnBotStopRequested(BotEvents.BotStartStopRequestEventArgs args)
        {

        }

        /// <summary>
        /// Called after the user pushes stop, and the bot has stopped.
        /// </summary>
        /// <param name="args"></param>
        static void BotEvents_OnBotStopped(EventArgs args)
        {

        }
        #endregion  

        #region Spec and Ring Stuff
        /// <summary>
        /// Updates the private variable currentlyEquippedRing with the ring you have equipped.
        /// </summary>
        public static void UpdateCurrentlyEquippedRing()
        {
            currentlyEquippedRing = null;

            List<String> validRings = new List<string>() { AgiDpsRingName, IntDpsRingName, HealingRingName, StrDpsRingName, TankRingName };

            if (validRings.Contains(Me.Inventory.Equipped.Finger1.Name)) { currentlyEquippedRing = Me.Inventory.Equipped.Finger1; }

            if (validRings.Contains(Me.Inventory.Equipped.Finger2.Name)) { currentlyEquippedRing = Me.Inventory.Equipped.Finger2; }


            if (currentlyEquippedRing == null)
            {
                Logger.Write(badColor, "You do not have the Legendary Ring Equipped!  Plugin will not work until a ring is equipped.");
            }

        }


        public static bool IsTankSpec()
        {
            if (Me.Specialization == WoWSpec.DeathKnightBlood
                || Me.Specialization == WoWSpec.DruidGuardian
                || Me.Specialization == WoWSpec.MonkBrewmaster
                || Me.Specialization == WoWSpec.PaladinProtection
                || Me.Specialization == WoWSpec.WarriorProtection)
            {
                return true;
            }
            return false;
        }
        public static bool IsHealSpec()
        {
            if (Me.Specialization == WoWSpec.DruidRestoration
                || Me.Specialization == WoWSpec.MonkMistweaver
                || Me.Specialization == WoWSpec.PaladinHoly
                || Me.Specialization == WoWSpec.PriestDiscipline
                || Me.Specialization == WoWSpec.PriestHoly
                || Me.Specialization == WoWSpec.ShamanRestoration)
            {
                return true;
            }
            return false;
        }
        public static bool IsAgiDpsSpec()
        {
            if (Me.Specialization == WoWSpec.DruidFeral
                || Me.Specialization == WoWSpec.HunterBeastMastery
                || Me.Specialization == WoWSpec.HunterMarksmanship
                || Me.Specialization == WoWSpec.HunterSurvival
                || Me.Specialization == WoWSpec.MonkWindwalker
                || Me.Specialization == WoWSpec.RogueAssassination
                || Me.Specialization == WoWSpec.RogueOutlaw
                || Me.Specialization == WoWSpec.RogueSubtlety
                || Me.Specialization == WoWSpec.ShamanEnhancement)
            {
                return true;
            }
            return false;
        }

        public static bool IsIntDpsSpec()
        {
            if (Me.Specialization == WoWSpec.DruidBalance
                || Me.Specialization == WoWSpec.MageArcane
                || Me.Specialization == WoWSpec.MageFire
                || Me.Specialization == WoWSpec.MageFrost
                || Me.Specialization == WoWSpec.PriestShadow
                || Me.Specialization == WoWSpec.ShamanElemental
                || Me.Specialization == WoWSpec.WarlockAffliction
                || Me.Specialization == WoWSpec.WarlockDemonology
                || Me.Specialization == WoWSpec.WarlockDestruction)
            {
                return true;
            }
            return false;
        }

        public static bool IsStrengthDpsSpec()
        {
            if (Me.Specialization == WoWSpec.DeathKnightFrost
                || Me.Specialization == WoWSpec.DeathKnightUnholy
                || Me.Specialization == WoWSpec.PaladinRetribution
                || Me.Specialization == WoWSpec.WarriorArms
                || Me.Specialization == WoWSpec.WarriorFury)
            {
                return true;
            }
            return false;
        }

        public static bool IsRingRightForSpec()
        {
            // no sense in checking if we have a null ring
            if (currentlyEquippedRing == null) { return false; }

            // tanking first
            if (IsTankSpec() && currentlyEquippedRing.Name != TankRingName)
            {
                return false;
            }

            // Healing
            if (IsHealSpec() && currentlyEquippedRing.Name != HealingRingName)
            {
                return false;
            }

            // Agi DPS
            if (IsAgiDpsSpec() && currentlyEquippedRing.Name != AgiDpsRingName)
            {
                return false;
            }

            // int DPS
            if (IsIntDpsSpec() && currentlyEquippedRing.Name != IntDpsRingName)
            {
                return false;
            }

            // strength dps
            if (IsStrengthDpsSpec() && currentlyEquippedRing.Name != StrDpsRingName)
            {
                return false;
            }

            return true;
        }

        public static bool IsEquippedRingReady()
        {
            if (currentlyEquippedRing == null)
            {
                if (frodoSettings.GeneralSettingDebugLogging)
                    Logger.Write(badColor, "Ring is not ready");
                return false;
            }
            if (currentlyEquippedRing.Usable == true && currentlyEquippedRing.CooldownTimeLeft.TotalMilliseconds <= 0)
            {
                if (frodoSettings.GeneralSettingDebugLogging)
                    Logger.Write(infoColor, "Ring is ready");
                return true;
            }
            return false;
        }
        #endregion

        #region Singular Stuff to make sure I am attacking the right things
        /// <summary>
        ///   Gets the nearby unfriendly units within specified range.  if no range specified, 
        ///   includes all unfriendly units
        /// </summary>
        /// <value>The nearby unfriendly units.</value>
        public static IEnumerable<WoWUnit> UnfriendlyUnits(int maxSpellDist = -1, WoWUnit origin = null)
        {
            if (origin == null)
                origin = StyxWoW.Me;

            List<WoWUnit> list = new List<WoWUnit>();
            List<WoWObject> objectList = ObjectManager.ObjectList;

            for (int i = 0; i < objectList.Count; i++)
            {
                Type type = objectList[i].GetType();
                if (type == typeof(WoWUnit) || type == typeof(WoWPlayer))
                {
                    WoWUnit t = objectList[i] as WoWUnit;
                    if (t != null && ValidUnit(t) && (maxSpellDist == -1 || origin.SpellDistance(t) < maxSpellDist))
                    {
                        list.Add(t);
                    }
                }
            }

            return list;
        }





        public static bool ValidUnit(WoWUnit p, bool showReason = false)
        {
            if (p == null || !p.IsValid)
                return false;

            // Ignore shit we can't select
            if (!p.CanSelect)
            {
                return false;
            }

            // Ignore shit we can't attack
            if (!p.Attackable)
            {
                return false;
            }

            // Duh
            if (p.IsDead)
            {
                return false;
            }

            // check for enemy players here as friendly only seems to work on npc's
            if (p.IsPlayer)
            {
                WoWPlayer pp = p.ToPlayer();
                if (pp.IsHorde == StyxWoW.Me.IsHorde && !pp.IsHostile)
                {
                    if (showReason)
                        return false;
                }

                return true;
            }



            // Ignore friendlies!
            if (p.IsFriendly)
            {
                return false;
            }

            // Dummies/bosses are valid by default. Period.
            if (p.IsTrainingDummy() || p.IsBoss())
                return true;

            // If it is a pet/minion/totem, lets find the root of ownership chain
            WoWPlayer pOwner = GetPlayerParent(p);

            // ignore if owner is player, alive, and not blacklisted then ignore (since killing owner kills it)
            // .. following .IsMe check to prevent treating quest mob summoned by us that we need to kill as invalid 
            if (pOwner != null && pOwner.IsAlive && !pOwner.IsMe)
            {
                if (!ValidUnit(pOwner))
                {
                    if (showReason)
                        return false;
                }
                if (!StyxWoW.Me.IsPvPFlagged)
                {
                    if (showReason)
                        return false;
                }
                if (Blacklist.Contains(pOwner, BlacklistFlags.Combat))
                {
                    if (showReason)
                        return false;
                }

                return true;
            }

            // And ignore non-combat pets
            if (p.IsNonCombatPet)
            {
                return false;
            }

            // And ignore critters (except for those ferocious ones or if set as BotPoi)
            if (p.IsIgnorableCritter())
            {
                return false;
            }
            return true;
        }
        public static WoWPlayer GetPlayerParent(WoWUnit unit)
        {
            // If it is a pet/minion/totem, lets find the root of ownership chain
            WoWUnit pOwner = unit;
            while (true)
            {
                if (pOwner.OwnedByUnit != null)
                    pOwner = pOwner.OwnedByRoot;
                else if (pOwner.CreatedByUnit != null)
                    pOwner = pOwner.CreatedByUnit;
                else if (pOwner.SummonedByUnit != null)
                    pOwner = pOwner.SummonedByUnit;
                else
                    break;
            }

            if (unit != pOwner && pOwner.IsPlayer)
                return pOwner as WoWPlayer;

            return null;
        }
        /// <summary>
        ///   Gets the nearby unfriendly units within 40 yards.
        /// </summary>
        /// <value>The nearby unfriendly units.</value>
        public static IEnumerable<WoWUnit> NearbyUnfriendlyUnits
        {
            get
            {
                return UnfriendlyUnits(40);
            }
        }



        public static IEnumerable<WoWUnit> UnitsInCombatWithMe(int maxSpellDist = -1)
        {
            return UnfriendlyUnits(maxSpellDist)
                .Where(
                    p => (p.Aggro || (p.Combat && p.TaggedByMe)) && p.CurrentTargetGuid == StyxWoW.Me.Guid);
        }

        public static IEnumerable<WoWUnit> NearbyUnitsInCombatWithMe
        {
            get { return NearbyUnfriendlyUnits.Where(p => p.Aggro || (p.Combat && p.CurrentTargetGuid == StyxWoW.Me.Guid)); }
        }
        #endregion

        #region Settings

        public static FrodoSettings LoadSettings()
        {
            FrodoSettings sett = new FrodoSettings();
            string settingsFile = System.IO.Path.Combine(Styx.Helpers.Settings.CharacterSettingsDirectory, PluginName + ".json");
            // load from file
            if (System.IO.File.Exists(settingsFile))
            {
                string jsonData = "";
                using (System.IO.StreamReader reader = new System.IO.StreamReader(settingsFile))
                {
                    jsonData = reader.ReadToEnd();
                }

                sett = JsonConvert.DeserializeObject<FrodoSettings>(jsonData);

            }
            else
            {
                // no settings file exists yet, so let's just take defaults (or what is already there).
                sett = new FrodoSettings();
                SaveSettings(sett, newFile: true);
                Logger.Write("Settings file missing.  Creating default settings.");
            }
            return sett;
        }


        public static void SaveSettings(FrodoSettings settings, bool newFile = false)
        {
            string settingsFile = System.IO.Path.Combine(Styx.Helpers.Settings.CharacterSettingsDirectory, PluginName + ".json");
            frodoSettings = settings;
            string saveData = JsonConvert.SerializeObject(settings);
            if (!string.IsNullOrEmpty(saveData))
            {
                using (System.IO.StreamWriter writer = new System.IO.StreamWriter(settingsFile))
                {
                    writer.Write(saveData);
                    if (newFile == false)
                    {
                        Logger.Write("Saved Settings!");
                    }
                }
                // anytime we save, we want to reload them back into the app.
                frodoSettings = LoadSettings();
            }
            else
            {
                Logger.Write("Failed to save settings!");
            }

        }
        #endregion
    }
}
