﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Diagnostics;

// Honorbuddy includes
using Styx;
using Styx.Common;
using Styx.CommonBot;
using Styx.Helpers;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Styx.Plugins;
using Styx.Pathing;
using Styx.WoWInternals.World;
using Newtonsoft.Json;


namespace Frodo
{
    public partial class Settings : Form
    {
        private FrodoSettings frodoSettings = null;

        public Settings()
        {
            InitializeComponent();
        }

        private void Settings_Load(object sender, EventArgs e)
        {
            // Set the version to display on the label
            Version ver = Frodo.PluginVersion;
            lblVersion.Text = String.Format("v{0}.{1}.{2}.{3}", ver.Major, ver.Minor, ver.Revision, ver.Build);
            rtfAbout.Rtf = Resource1.Document;
            frodoSettings = Frodo.LoadSettings();

            if (frodoSettings != null)
            {
                GeneralSettingDebugLogging.Checked = frodoSettings.GeneralSettingDebugLogging;

                TankSettingUseOnCooldown.Checked = frodoSettings.TankSettingUseOnCooldown;
                TankSettingHealthPercent.Value = frodoSettings.TankSettingHealthPercent;
                TankSettingNearbyUnitsInCombat.Value = frodoSettings.TankSettingNearbyUnitsInCombat;
                //TankSettingOnlyOnBoss.Checked = frodoSettings.TanksSettingOnlyOnBoss;
                TankSettingUseOnBoss.Checked = frodoSettings.TankSettingUseOnBoss;

                DpsSettingNearbyUnitsInCombat.Value = frodoSettings.DpsSettingNearbyUnitsInCombat;
                //DpsSettingOnlyOnBoss.Checked = frodoSettings.DpsSettingOnlyOnBoss;
                DpsSettingUseOnBoss.Checked = frodoSettings.DpsSettingUseOnBoss;
                DpsSettingUseOnCooldown.Checked = frodoSettings.DpsSettingUseOnCooldown;

                HealerSettingUseOnCooldown.Checked = frodoSettings.HealSettingUseOnCooldown;
            }
            else
            {
                MessageBox.Show("There's an issue with the Frodo Settings.  They came back as null");
            }

        }

        private void Settings_Dispose(object sender, EventArgs e)
        {

            FrodoSettings sett = frodoSettings;

            sett.GeneralSettingDebugLogging = GeneralSettingDebugLogging.Checked;

            sett.TankSettingUseOnCooldown = TankSettingUseOnCooldown.Checked;
            sett.TankSettingHealthPercent = (int)TankSettingHealthPercent.Value;
            sett.TankSettingNearbyUnitsInCombat = (int)TankSettingNearbyUnitsInCombat.Value;
            //sett.TanksSettingOnlyOnBoss = TankSettingOnlyOnBoss.Checked;
            sett.TankSettingUseOnBoss = TankSettingUseOnBoss.Checked;

            sett.DpsSettingNearbyUnitsInCombat = (int)DpsSettingNearbyUnitsInCombat.Value;
            //sett.DpsSettingOnlyOnBoss = DpsSettingOnlyOnBoss.Checked;
            sett.DpsSettingUseOnBoss = DpsSettingUseOnBoss.Checked;
            sett.DpsSettingUseOnCooldown = DpsSettingUseOnCooldown.Checked;

            sett.HealSettingUseOnCooldown = HealerSettingUseOnCooldown.Checked;

            // Save user settings
            Frodo.SaveSettings(sett);


        }


        /// <summary>
        /// Custom form drag and close handler.
        /// </summary>
        Point FormHeaderCursorPosition;
        private void FormHeader_MouseDown(object sender, MouseEventArgs e) { FormHeaderCursorPosition = new Point(-e.X, -e.Y); }
        private void FormHeader_MouseMove(object sender, MouseEventArgs e) { if (e.Button == MouseButtons.Left) { Point mousePos = Control.MousePosition; mousePos.Offset(FormHeaderCursorPosition.X, FormHeaderCursorPosition.Y); Location = mousePos; } }
        private void FormHeaderClose_Click(object sender, EventArgs e) { this.Dispose(); }

        /// <summary>
        /// Resize a borderless form. It does have flicker on resize, but the form sizes.
        /// </summary>
        /// <param name="m"></param>
        protected override void WndProc(ref Message m)
        {
            const int wmNcHitTest = 0x84;
            const int htLeft = 10;
            const int htRight = 11;
            const int htTop = 12;
            const int htTopLeft = 13;
            const int htTopRight = 14;
            const int htBottom = 15;
            const int htBottomLeft = 16;
            const int htBottomRight = 17;

            if (m.Msg == wmNcHitTest)
            {
                int x = (int)(m.LParam.ToInt64() & 0xFFFF);
                int y = (int)((m.LParam.ToInt64() & 0xFFFF0000) >> 16);
                Point pt = PointToClient(new Point(x, y));
                Size clientSize = ClientSize;
                ///allow resize on the lower right corner
                if (pt.X >= clientSize.Width - 16 && pt.Y >= clientSize.Height - 16 && clientSize.Height >= 16)
                {
                    m.Result = (IntPtr)(IsMirrored ? htBottomLeft : htBottomRight);
                    return;
                }
                ///allow resize on the lower left corner
                if (pt.X <= 16 && pt.Y >= clientSize.Height - 16 && clientSize.Height >= 16)
                {
                    m.Result = (IntPtr)(IsMirrored ? htBottomRight : htBottomLeft);
                    return;
                }
                ///allow resize on the upper right corner
                if (pt.X <= 16 && pt.Y <= 16 && clientSize.Height >= 16)
                {
                    m.Result = (IntPtr)(IsMirrored ? htTopRight : htTopLeft);
                    return;
                }
                ///allow resize on the upper left corner
                if (pt.X >= clientSize.Width - 16 && pt.Y <= 16 && clientSize.Height >= 16)
                {
                    m.Result = (IntPtr)(IsMirrored ? htTopLeft : htTopRight);
                    return;
                }
                ///allow resize on the top border
                if (pt.Y <= 16 && clientSize.Height >= 16)
                {
                    m.Result = (IntPtr)(htTop);
                    return;
                }
                ///allow resize on the bottom border
                if (pt.Y >= clientSize.Height - 16 && clientSize.Height >= 16)
                {
                    m.Result = (IntPtr)(htBottom);
                    return;
                }
                ///allow resize on the left border
                if (pt.X <= 16 && clientSize.Height >= 16)
                {
                    m.Result = (IntPtr)(htLeft);
                    return;
                }
                ///allow resize on the right border
                if (pt.X >= clientSize.Width - 16 && clientSize.Height >= 16)
                {
                    m.Result = (IntPtr)(htRight);
                    return;
                }
            }
            base.WndProc(ref m);
        }

        /// <summary>
        /// Opens the purchase URL. Use the 'tag' field on btnPurchase to set the url location.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPurchase_Click(object sender, EventArgs e)
        {
            string url = Convert.ToString(btnPurchase.Tag);
            System.Diagnostics.Process.Start(url);
        }

        /// <summary>
        /// Opens the support URL. Use the 'tag' field on btnSupport to set the url location.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSupport_Click(object sender, EventArgs e)
        {
            string url = Convert.ToString(btnPurchase.Tag);
            System.Diagnostics.Process.Start(url);
        }

        private void TankSettingUseOnCooldown_CheckedChanged(object sender, EventArgs e)
        {
            //FrodoSettings tempSetting = Frodo.frodoSettings;
            //tempSetting.TankSettingUseOnCooldown = TankSettingUseOnCooldown.Checked;
            //Frodo.frodoSettings = tempSetting;

            TankSettingNearbyUnitsInCombat.Enabled = !TankSettingUseOnCooldown.Checked;
            TankSettingHealthPercent.Enabled = !TankSettingUseOnCooldown.Checked;
        }

        private void GeneralSettingDebugLogging_CheckedChanged(object sender, EventArgs e)
        {
            frodoSettings.GeneralSettingDebugLogging = GeneralSettingDebugLogging.Checked;
        }

        private void FormHeaderClose_Click_1(object sender, EventArgs e)
        {
            Dispose(true);
        }

        private void TankSettingNearbyUnitsInCombat_ValueChanged(object sender, EventArgs e)
        {
            frodoSettings.TankSettingNearbyUnitsInCombat = (int)TankSettingNearbyUnitsInCombat.Value;
        }

        private void TankSettingHealthPercent_ValueChanged(object sender, EventArgs e)
        {
            frodoSettings.TankSettingHealthPercent = (int)TankSettingHealthPercent.Value;
        }
    }
}
